--[[
Spezialisierung die die Standardlichter beim Fahrzeug sichtbar macht.

Author:		Ifko[nator]
Datum:		10.02.2016
Version:	v1.9

History:	v1.0 @ 11.08.2015 - initial implementation
			v1.1 @ 12.08.2015 - support for more as one trailer. The brake lights are now on all attached trailers visible, if the attacher vehicle is not movig.
			V1.2 @ 24.09.2015 - support for all attachables, wich have no brake lights, for example: weights.
			V1.5 @ 23.11.2015 - last fixes for public release
			V1.8 @ 10.02.2016 - mp fix
			V1.9 @ 18.07.2016 - changed script functionality to activate all lights with lightType 0 when vehicle is active
]]

DrivingLightsExtension = {};

function DrivingLightsExtension.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization((Drivable or Attachable), specializations)
end;

function DrivingLightsExtension:load(xmlFile)
	self.standDrivingLights = {};
	local numStandDrivingLights = 0;

	local lights1 = string.format("vehicle.lights.light(%d)", numStandDrivingLights);
	local lights2 = string.format("vehicle.lightCoronas.lightCorona(%d)", numStandDrivingLights);
	
	-- If self has lights
	if hasXMLProperty(xmlFile, lights1) then
		while true do
			local lights1 = string.format("vehicle.lights.light(%d)", numStandDrivingLights);
			if not hasXMLProperty(xmlFile, lights1) then
				break;
			end;
			
			local standDrivingLightOriginal = Utils.indexToObject(self.components, getXMLString(xmlFile, lights1 .. "#decoration"));
			local standLightType = Utils.getNoNil(getXMLInt(xmlFile, lights1 .. "#lightType"), 9)
				
			if standDrivingLightOriginal ~= nil and standLightType == 0 then
				--## simply clone the light.
				local standDrivingLight = clone(standDrivingLightOriginal, true, false, false);
				setVisibility(standDrivingLight, false);
				table.insert(self.standDrivingLights, standDrivingLight);
			end;
			
			numStandDrivingLights = numStandDrivingLights + 1;
		end;
	end;
	
	local numStandDrivingLights = 0;
	
	-- if self has lightCoronas
	if hasXMLProperty(xmlFile, lights2) then
		while true do
			local parameter = string.format("vehicle.lightCoronas.lightCorona(%d)", numStandDrivingLights);
			if not hasXMLProperty(xmlFile, parameter) then
				break;
			end;
			
			local standDrivingLightOriginal = Utils.indexToObject(self.components, getXMLString(xmlFile, parameter .. "#index"));
						
			if standDrivingLightOriginal ~= nil then
				--## simply clone the lightCorona.
				local standDrivingLight = clone(standDrivingLightOriginal, true, false, false);
				setVisibility(standDrivingLight, false);
				table.insert(self.standDrivingLights, standDrivingLight);
			end;
			
			numStandDrivingLights = numStandDrivingLights + 1;
		end;
	end;
	
	self.doDriving = false;
end;

function DrivingLightsExtension:update(dt)
	for _, standDrivingLight in pairs(self.standDrivingLights) do
		if self:getIsActive() then
			if SpecializationUtil.hasSpecialization(Drivable, self.specializations) then

				
				local someoneIsInTheVehicle; 
				
				if self.cp ~= nil and self.modFM == nil then
					someoneIsInTheVehicle = self.cp.isDriving or self.isEntered or self.isControlled or self.isHired;
				elseif self.cp ~= nil and self.modFM ~= nil then
					someoneIsInTheVehicle = self.cp.isDriving or self.modFM.FollowState ~= FollowMe.STATE_NONE or self.isEntered or self.isControlled or self.isHired;
				elseif self.cp == nil and self.modFM ~= nil then
					someoneIsInTheVehicle = self.modFM.FollowState ~= FollowMe.STATE_NONE or self.isEntered or self.isControlled or self.isHired;
				else
					someoneIsInTheVehicle = self.isEntered or self.isControlled or self.isHired; 
				end;
	
				--## moving direction is much better as the wheel speed for this
				
				local movingDirection;
				
				if self.isEntered then
					movingDirection = InputBinding.getDigitalInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE);
					
					if InputBinding.isAxisZero(movingDirection) then
						--## controller
						movingDirection = InputBinding.getAnalogInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE);
					end;
				else
					movingDirection = 0;
				end;
				
				local cruiseControlIsOff;
				
				if self.isHired then
					if self.isHirableBlocked then
						if self.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF then
							cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
						else
							cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_ACTIVE;
						end;
					else
						cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
					end;
				elseif self.cp ~= nil and self.cp.isDriving then
					if not self.cp.TrafficDriving then
						if self.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF then
							cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
						else
							cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_ACTIVE;
						end;
					else
						cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
					end;
				else
					cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
				end;
				
					setVisibility(standDrivingLight, true);
					self.doDriving = true;

			elseif SpecializationUtil.hasSpecialization(Attachable, self.specializations) then
					setVisibility(standDrivingLight, true);
					self.doDriving = true;
			end;
		else
			setVisibility(standDrivingLight, false);
		end;
	end;
	
	--## support for all attachables, wich have no driving lights, for example: weights.
	if SpecializationUtil.hasSpecialization(Attachable, self.specializations) then
		if #self.standDrivingLights == 0 then
			if self.attacherVehicle ~= nil and self.attacherVehicle:getIsActive() and self.attacherVehicle.doDriving then
				self.doDriving = true;
			else
				self.doDriving = false;
			end;
		end;
	end;
end;

function DrivingLightsExtension:draw()end;
function DrivingLightsExtension:delete()end;
function DrivingLightsExtension:mouseEvent(posX, posY, isDown, isUp, button)end;
function DrivingLightsExtension:keyEvent(unicode, sym, modifier, isDown)end;